import java.util.Collection;
import java.util.ArrayList;
import java.util.LinkedList;

// See http://Yost.com/computers/java/closures
// See http://www.martinfowler.com/bliki/Closure.html
public class FowlerDemo {

  public static void main(String[] args) throws Exception {
    emps = new ArrayList<Employee>();
    Employee john = new Employee("john", true, 200);
    emps.add(john);
    emps.add(new Employee("joe",  false, 40));

    System.out.println(managers(emps));

    System.out.println(highPaid(emps));

    Selector<Employee> highPaid = paidMore(150);
    System.out.println(highPaid.select(john));
  }

  static Collection<Employee> emps;

  static Collection<Employee> managers(Collection<? extends Employee> empsArg) {
//  return do(empsArg) Selector<Employee>.select(Employee item) { return item.isManager; }.selectAll();
    return new $Selector1<Employee>(empsArg).selectAll(); // Works like this
  }

  static Collection<Employee> highPaid(Collection<? extends Employee> empsArg) {
    int threshold = 150;
//  return do(empsArg, threshold) Selector<Employee>.select(Employee item) { return item.salary < threshold; }.selectAll();
    return new $Selector2<Employee>(empsArg, threshold).selectAll(); // Works like this
  }

  static Selector<Employee> paidMore(int amount) {
//  return do(emps, amount) Selector<Employee>.select(Employee item) { return item.salary > amount; };
    return new $Selector3<Employee>(emps, amount); // Works like this
  }

  //----------------------------------------------------------------------------
  // Code from here to end of class is generated by the compiler

  private static class $Selector1<T> extends Selector<Employee> {

    public $Selector1(Collection<? extends Employee> c) {
      super(c);
    }

    public boolean select(Employee item) {
      return item.isManager;
    }
  }

  private static class $Selector2<T> extends Selector<Employee> {

    public $Selector2(Collection<? extends Employee> c, int threshold) {
      super(c);
      this.threshold = threshold;
    }

    private final int threshold;

    public boolean select(Employee item) {
      return item.salary < threshold;
    }
  }

  private static class $Selector3<T> extends Selector<Employee> {

    public $Selector3(Collection<? extends Employee> c, int amount) {
      super(c);
      this.amount = amount;
    }

    private final int amount;

    public boolean select(Employee item) {
      return item.salary > amount;
    }
  }
}

class Employee {

  Employee(String name, boolean isManager, int salary) {
    this.name = name;
    this.isManager = isManager;
    this.salary = salary;
  }

  final String  name;
  final boolean isManager;
  final int     salary;

  public String toString() { return name; }
}

//==============================================================================
// Probable candidates for java.util

abstract class Selector<T> {

  public Selector(Collection<? extends T> c) {
    this.c = c;
  }

  private final Collection<? extends T> c;

  public abstract boolean select(T item);

  public Collection<T> selectAll() {
    return Selections.selectAll(c, this);
  }

  public Collection<T> selectAll(Collection<T> result) {
    return Selections.selectAll(c, this, result);
  }
}

class Selections {

  private Selections() {}

  public static <T> Collection<T> selectAll(Collection<? extends T> c,
                                            Selector<T> selector) {
    Collection<T> result = new LinkedList<T>();
    return selectAll(c, selector, result);
  }

  public static <T> Collection<T> selectAll(Collection<? extends T> c,
                                            Selector<T> selector,
                                            Collection<T> result) {
    for (T obj : c) {
      if (selector.select(obj)) {
        result.add(obj);
      }
    }
    return result;
  }
}
